import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { CompetencytypeModel, MyCompetencytypeModel } from 'src/app/shared/model/competencytype.model';
import { CompetencytypeService } from 'src/app/shared/services/competencytype.service';
import { FileService } from 'src/app/shared/services/file.service';
export interface DataModel {
  id: string
  companyId: string
  edesc: string
  name: string
  code: string
  level: string
  weight: number
  checked: boolean
}
@Component({
  selector: 'app-type-registration',
  templateUrl: './type-registration.component.html',
  styleUrls: ['./type-registration.component.scss']
})
export class TypeRegistration {
  isChecked: boolean = false;
  currentPage = 1
  selectedItems: string[] = [];
  page = Array.from({ length: 1 }, (_, i) => i + 1);

  numDataListChecked = 0
  isDataListChecked = false
  isDataListCheckedAll = false

  dataSelectList: DataModel[] = [];
  dataLoading = false
  dataSelect: DataModel = { id: "", companyId: "", edesc: "", name: "", code: "", level: "", weight: 0.00, checked: false }
  competoncy_typeList: { check: boolean; data: DataModel & { checked?: boolean } }[] = []
  competoncy_type: CompetencytypeModel = new MyCompetencytypeModel({})
  modalStatus: 'add' | 'edit' | 'delete' | 'deleteGroup' = 'add'
  search = ""
  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';

  constructor(private competencytypeService: CompetencytypeService,
    private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private fileService: FileService

  ) { }
  ngOnInit(): void {
    this.getCompetencytypeList()
  }

  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.dataLoading = true
    this.fileService.uploadExcel(formData, 'competency_type').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getCompetencytypeList()
        } else {
          this.showAlert(response.message, 'error')
          this.dataLoading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    })
  }

  downloadFile() {
    const fileName = 'IMPORT_COMPETENCY_TYPE.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }

  getCompetencytypeList() {
    this.dataLoading = true
    this.competencytypeService.getList().subscribe({
      next: response => {
        this.competoncy_typeList = response.map(x => ({ check: false, data: { id: x.competencyTypeId, companyId: x.companyId, edesc: x.edesc, name: x.tdesc, code: x.shortName, weight: x.weight, level: x.expectationLevel, checked: false } }))
        this.dataLoading = false
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.searchChange()
        this.cdr.detectChanges()
      }, error: error => {
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    })
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.dataListFilter().length / 10) }, (_, i) => i + 1);
    this.dataListCheck()
  }
  dataListFilter() {
    return this.competoncy_typeList.filter(x => {
      const data = x.data
      const match = data.id.toLowerCase().includes(this.search.toLowerCase()) || data.name.toLowerCase().includes(this.search.toLowerCase()) || data.code.toLowerCase().includes(this.search.toLowerCase());
      return match;
    });
  }
  setData(data?: DataModel) {
    this.dataSelect = JSON.parse(JSON.stringify(data || { id: "", companyId: "", name: "", edesc: "", code: "", level: "", checked: false }));
  }
  // ฟังก์ชันสำหรับการเพิ่ม ลบ หรือแก้ไข ข้อมูล
  addType_registration() {
    const body = new MyCompetencytypeModel({ competencyTypeId: this.dataSelect.id, tdesc: this.dataSelect.name, edesc: this.dataSelect.edesc, shortName: this.dataSelect.code, weight: +(this.dataSelect.weight), expectationLevel: this.dataSelect.level })
    this.dataLoading = true
    this.competencytypeService.post(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getCompetencytypeList()
        } else {
          this.showAlert(response.message, 'error')
          this.dataLoading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    })
  }
  deleteType_registration() {
    let body: CompetencytypeModel | CompetencytypeModel[] = [];

    if (this.dataSelect.id) {
      body = new MyCompetencytypeModel({ competencyTypeId: this.dataSelect.id, tdesc: this.dataSelect.name, edesc: this.dataSelect.edesc, shortName: this.dataSelect.code, weight: +(this.dataSelect.weight), expectationLevel: this.dataSelect.level });
    } else {
      body = this.competoncy_typeList.filter(x => x.check).map(x => new MyCompetencytypeModel({ competencyTypeId: x.data.id, tdesc: x.data.name, edesc: x.data.edesc, shortName: x.data.code, weight: +(this.dataSelect.weight), expectationLevel: x.data.level }));
    }
    this.dataLoading = true
    this.competencytypeService.delete(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success');
          this.getCompetencytypeList();
        } else {
          this.showAlert(response.message, 'error');
          this.dataLoading = false
          this.cdr.detectChanges()
        }
      },
      error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    });
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }


  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.competoncy_typeList.filter(x => {
      const data = x.data
      const match = data.id.toLowerCase().includes(this.search.toLowerCase()) || data.name.toLowerCase().includes(this.search.toLowerCase()) || data.code.toLowerCase().includes(this.search.toLowerCase());
      return match;
    }).forEach(x => x.check = selectAll);
    this.dataListCheck();
  }

  dataListCheck() {
    const dataCheck = this.dataListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.competoncy_typeList.filter(x => x.check).length;
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }
  clearData(modalStatus: string) {
    if (modalStatus == 'add') {
      this.setData()
    } else if (modalStatus == 'edit') {
      this.setData({ id: this.dataSelect.id, companyId: "", name: "", edesc: "", code: "", level: "", weight: 0.00, checked: this.dataSelect.checked })
    }
  }
}

