import { ChangeDetectorRef, Component } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { MyEmeventgrpModel } from 'src/app/shared/model/emeventgrp.model';
import { EventgrpModel, MyEventgrpModel, } from 'src/app/shared/model/eventgrp.model';
import { PmsWorkingTimeModel, MyPmsWorkingTimeModel } from 'src/app/shared/model/pms-working-time.model';
import { EventgrpService } from 'src/app/shared/services/eventgrp.service';
import { FileService } from 'src/app/shared/services/file.service';
import { PmsWorkingTimeService } from 'src/app/shared/services/pms-working-time.service';
interface table {
    currentPage: number,
    page: number[],
    search: string
}
@Component({
    selector: 'app-time-attendance',
    templateUrl: './time-attendance.component.html',
    styleUrls: ['./time-attendance.component.scss']
})
export class TimeAttendanceComponent {
    pathTitle = ['การประเมินจัดการประสิทธิภาพ', 'ทะเบียนการประเมินเวลาทำงาน', 'การจัดการหัวข้อการประเมินเวลาทำงาน']

    numDataListChecked = 0
    isDataListChecked = false
    isDataListCheckedAll = false
    currentPage = 1;
    page = Array.from({ length: 1 }, (_, i) => i + 1);
    search = '';
    modalStatus: "add" | "edit" | "delete" | "deleteGroup" = "add"
    modal: table = {
        currentPage: 1,
        page: Array.from({ length: 1 }, (_, i) => i + 1),
        search: ""
    }


    eventgrp: { loading: boolean, selectIndex: number, selectList: EventgrpModel[], dataList: { check: boolean, data: EventgrpModel }[] } = { loading: false, selectIndex: -1, selectList: [new MyEventgrpModel()], dataList: [] }
    pmsWorkingTime: { loading: boolean, select: PmsWorkingTimeModel, dataList: { check: boolean, data: PmsWorkingTimeModel }[] } = { loading: false, select: new MyPmsWorkingTimeModel(), dataList: [] }
    constructor(
        private toastr: ToastrService,
        private cdr: ChangeDetectorRef,
        private fileService: FileService,
        private eventgrpService: EventgrpService,
        private pmsWorkingTimeService: PmsWorkingTimeService,
    ) { }

    ngOnInit(): void {
        this.getEventgrpList()
        this.getPmsWorkingTimeList()
    }
    getPmsWorkingTimeList() {
        this.pmsWorkingTime.loading = true
        this.pmsWorkingTimeService.getList().subscribe({
            next: response => {
                this.pmsWorkingTime.dataList = response.map(x => ({ check: false, data: new MyPmsWorkingTimeModel(x) }))
                this.pmsWorkingTime.loading = false
                this.isDataListCheckedAll = false
                this.dataListCheckAll()
                this.searchChange()
                this.cdr.detectChanges();
            }, error: error => {
                this.pmsWorkingTime.loading = false
                this.cdr.detectChanges()
            }
        })
    }
    pmsWorkingTimeFilter() {
        return this.pmsWorkingTime.dataList.filter((x) => {
            const data = x.data
            const match = data.pmsWorkingTimeId.toLowerCase().includes(this.search.toLowerCase()) ||
                data.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
                data.detailUnit.toLowerCase().includes(this.search.toLowerCase()) ||
                (data.weight + "").toLowerCase().includes(this.search.toLowerCase())
            return match;
        });
    }
    searchChange() {
        this.currentPage = 1;
        const filteredData = this.pmsWorkingTimeFilter();
        this.page = Array.from({ length: Math.ceil(filteredData.length / 10) }, (_, i) => i + 1);
        this.dataListCheck();
    }
    selectData(data?: PmsWorkingTimeModel) {
        this.pmsWorkingTime.select = new MyPmsWorkingTimeModel(data)
    }
    dataListCheckAll() {
        const selectAll = this.isDataListCheckedAll;
        this.pmsWorkingTimeFilter().forEach(x => x.check = selectAll);
        this.dataListCheck();
    }
    dataListCheck() {
        const dataCheck = this.pmsWorkingTimeFilter();
        this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
        this.numDataListChecked = this.pmsWorkingTime.dataList.filter(x => x.check).length;
        this.isDataListChecked = Boolean(this.numDataListChecked)
    }

    getEventgrpList() {
        this.eventgrp.loading = true
        this.eventgrpService.getList().subscribe({
            next: response => {
                this.eventgrp.dataList = response.map(x => ({ check: false, data: new MyEventgrpModel(x) }))
                this.eventgrp.loading = false
                this.isDataListCheckedAll = false
                this.dataListCheckAll()
                this.searchChange()
                this.cdr.detectChanges();
            }, error: error => {
                this.eventgrp.loading = false
                this.cdr.detectChanges()
            }
        })
    }
    eventgrpListFilter() {
        return this.eventgrp.dataList.filter(x => {
            const data = x.data
            const match = data.eventgrpId.toLowerCase().includes(this.modal.search.toLowerCase()) || data.tdesc.toLowerCase().includes(this.modal.search.toLowerCase()) || data.edesc.toLowerCase().includes(this.modal.search.toLowerCase());
            return match;
        })
    }
    selectEventgrp(data?: EventgrpModel) {
        if (!data) {
            this.pmsWorkingTime.select.emeventgrp.splice(this.eventgrp.selectIndex, 1);
            return;
        }
        this.pmsWorkingTime.select.emeventgrp.push(new MyEmeventgrpModel(data))
    }


    clearData(modalStatus: string) {
        if (modalStatus == 'add') {
            this.pmsWorkingTime.select = new MyPmsWorkingTimeModel()
        } else if (modalStatus == 'edit') {
            this.pmsWorkingTime.select = new MyPmsWorkingTimeModel({ pmsWorkingTimeId: this.pmsWorkingTime.select.pmsWorkingTimeId })
        }
        this.eventgrp.selectList = [new MyEventgrpModel()]
    }

    modalSearchChange(dataList: any) {
        this.modal.currentPage = 1
        this.modal.page = Array.from({ length: Math.ceil(dataList.length / 10) }, (_, i) => i + 1);
    }
    updatePmsWorkingTime() {
        this.pmsWorkingTime.loading = true
        this.pmsWorkingTimeService.post(new MyPmsWorkingTimeModel(this.pmsWorkingTime.select)).subscribe({
            next: response => {
                if (response.success) {
                    this.showAlert(response.message, 'success')
                    this.getPmsWorkingTimeList()
                    this.searchChange()
                } else {
                    this.showAlert(response.message, 'error')
                    this.pmsWorkingTime.loading = false
                }
                this.cdr.detectChanges()
            }, error: error => {
                this.showAlert(error.message, 'error')
                this.pmsWorkingTime.loading = false
                this.cdr.detectChanges()
            }
        })
    }
    deletePmsWorkingTime() {
        let body: PmsWorkingTimeModel[] = []
        if (this.modalStatus == "deleteGroup") {
            body = this.pmsWorkingTime.dataList.filter(x => x.check).map(x => new MyPmsWorkingTimeModel(x.data))
        } else {
            body = [new MyPmsWorkingTimeModel(this.pmsWorkingTime.select)]
        }
        this.pmsWorkingTime.loading = true
        this.pmsWorkingTimeService.delete(body).subscribe({
            next: response => {
                if (response.success) {
                    this.showAlert(response.message, 'success')
                    this.getPmsWorkingTimeList()
                } else {
                    this.showAlert(response.message, 'error')
                    this.pmsWorkingTime.loading = false
                    this.cdr.detectChanges()
                }
            }, error: error => {
                this.showAlert(error.message, 'error')
                this.pmsWorkingTime.loading = false
                this.cdr.detectChanges()
            }
        })
    }
    showAlert(text: string, type: 'success' | 'error') {
        this.toastr[type](text, 'แจ้งเตือน', {
            timeOut: 3000,
            positionClass: 'toast-top-right',
        })
    }
}
